@@ -1,6 +1,6 @@
 
 /*
- * $Header: /cvs/gnome/gperfmeter/src/resources.c,v 1.31 2004/11/15 17:12:49 richb Exp $
+ * $Header: /cvs/gnome/gperfmeter/src/resources.c,v 1.36 2005/01/21 20:05:03 richb Exp $
  *
  * Copyright (c) 1990-2004 Sun Microsystems, Inc. All Rights Reserved.
  *
@@ -30,7 +30,6 @@
 #include <string.h>
 #include <pwd.h>
 #include <errno.h>
-#include <popt.h>
 #include <gnome.h>
 #include <gconf/gconf-client.h>
 #include <gconf/gconf-value.h>
@@ -40,7 +39,7 @@
 #endif
 
 static int optTmpVal;
-static poptContext ctx;
+static GOptionContext *ctx;
 
 /* flag added to check if we are dealing with command line options */
 static gboolean cmd_line;
@@ -67,74 +66,81 @@ static PmARStruct AR;
 /* globals */
 
 /* prototypes */
+static gboolean pm_arguments_post_parse(GOptionContext *, GOptionGroup *, 
+                                        gpointer, GError **);
+
 static void InitResources(void);
 static void ParseCmdOptions(int, char **);
 static void PostParseOptions(void);
 static void PmCheckLimits(void);
 static void Usage(void);
-static void parse_popt_args(poptContext, enum poptCallbackReason,
-                            const struct poptOption *, const char *, void *);
 
 static int save_session(GnomeClient *, gint, GnomeRestartStyle,
                         gint, GnomeInteractStyle, gint, gpointer);
 static gint client_die(GnomeClient *, gpointer);
 
 static char *color_to_string(const GdkColor *);
+gchar *pm_log_create_tmp_filename(void);
 
-static struct poptOption options[] = {
-  { NULL, '\0', POPT_ARG_CALLBACK, &parse_popt_args, 
-    0, NULL },
+gboolean opt_all, opt_panel, opt_log, opt_debug, opt_horizontal, opt_line,
+         opt_solid, opt_version, opt_vertical;
+gchar *opt_logfile = NULL, *opt_meter = NULL, *opt_ceiling = NULL,
+      *opt_maxvalue = NULL, **opt_hostnames = NULL;
+gint opt_pagelength, opt_sampletime;
 
-  { "all", 'a',  POPT_ARG_NONE,     NULL, 0, 
+static GOptionEntry entries[] =
+{
+  { "all", 'a', 0, G_OPTION_ARG_NONE, &opt_all, 
     N_("Display all the performance statistic meters simultaneously"), NULL },
 
-  { "panel", 'f',  POPT_ARG_NONE,     NULL, 0, 
+  { "panel", 'f', 0, G_OPTION_ARG_NONE, &opt_panel, 
     N_("Display the performance meter in the front panel"), NULL },
 
-  { "log", 'l',  POPT_ARG_NONE,     NULL, 0, 
+  { "log", 'l', 0, G_OPTION_ARG_NONE, &opt_log, 
     N_("Turn statistic logging on"), NULL },
 
-  { "logfile", 'n',  POPT_ARG_STRING,   NULL, 0, 
+  { "logfile", 'n', 0, G_OPTION_ARG_FILENAME, &opt_logfile, 
     N_("Name of file to log statistics to"), N_("SAMPLE_FILENAME") },
 
-  { "pagelength", 'p',  POPT_ARG_INT,      &optTmpVal, 0, 
+  { "pagelength", 'p', 0, G_OPTION_ARG_INT, &opt_pagelength,
     N_("Page length for samples written to the log file"), N_("PAGE_LENGTH") },
 
-  { "sampletime", 's',  POPT_ARG_INT,      &optTmpVal, 0, 
+  { "sampletime", 's', 0, G_OPTION_ARG_INT, &opt_sampletime,
     N_("Set the sample time interval in seconds"), N_("SAMPLE_TIME") },
 
-  { "meter", 't',  POPT_ARG_STRING,   NULL, 0, 
+  { "meter", 't', 0, G_OPTION_ARG_STRING, &opt_meter, 
     N_("Specify a meter to display"), N_("METER") },
 
-  { "ceiling", 'C',  POPT_ARG_STRING,   NULL, 0, 
+  { "ceiling", 'C', 0, G_OPTION_ARG_STRING, &opt_ceiling, 
     N_("Set a ceiling value for a given meter"), N_("\"METER CEILING\"") },
 
-  { "debug", 'D',  POPT_ARG_NONE,     NULL, 0, 
+  { "debug", 'D', 0, G_OPTION_ARG_NONE, &opt_debug, 
     N_("Turn debugging on"), NULL },
 
-  { "horizontal", 'H',  POPT_ARG_NONE,     NULL, 0, 
+  { "horizontal", 'H', 0, G_OPTION_ARG_NONE, &opt_horizontal, 
     N_("Display meters in a horizontal orientation"), NULL },
 
-  { "line", 'L',  POPT_ARG_NONE,     NULL, 0, 
+  { "line", 'L', 0, G_OPTION_ARG_NONE, &opt_line, 
     N_("Display the meters as line strip charts"), NULL },
 
-  { "solid", 'S',  POPT_ARG_NONE,     NULL, 0, 
+  { "solid", 'S', 0, G_OPTION_ARG_NONE, &opt_solid, 
     N_("Display the meters as solid line strip charts"), NULL },
 
-  { "version", 'v',  POPT_ARG_NONE,     NULL, 0, 
+  { "version", 'v', 0, G_OPTION_ARG_NONE, &opt_version, 
     N_("Show version number"), NULL },
 
-  { "vertical", 'V',  POPT_ARG_NONE,     NULL, 0, 
+  { "vertical", 'V', 0, G_OPTION_ARG_NONE, &opt_vertical, 
     N_("Display meters in a vertical orientation"), NULL },
 
-  { "maxvalue", 'M',  POPT_ARG_STRING,   NULL, 0, 
+  { "maxvalue", 'M', 0, G_OPTION_ARG_STRING, &opt_maxvalue, 
     N_("Set a current, minimum and maximum value for the given meter"),
     N_("\"METER CURMAX MINMAX MAXMAX\"") },
 
-  { NULL, '\0', 0,                 NULL, 0, 
-    NULL, NULL }
-};
+  { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_hostnames,
+      "Show statistics for a specific computer", NULL },
 
+  { NULL }
+};
 
 /* Load the application resource Data
  *
@@ -183,7 +189,8 @@ PmLoadResources(int argc, char **argv)
     PM->LoadByHost          = AR.LoadByHost;
 
     PM->IsRemote = FALSE;
-    PmStrcpy(PM->Hostname, "");
+    PM->Hostname = g_strdup("localhost");
+    PM->LogFilename = pm_log_create_tmp_filename();
 
     PM->MachineNames = AR.MachineNames;
     PM->NumMachineNames = 0;
@@ -465,6 +472,7 @@ InitResources()
      */
     PM->CanvasColor =  grey; 
     PM->LabelColor = black; 
+    PM->ShowHostName = FALSE;
     PM->ShowMenubar = TRUE;
     PM->ShowStripChart = TRUE;
     PM->ShowLiveActionBar = TRUE;
@@ -500,267 +508,246 @@ PmCheckLimits()
 
 
 /*ARGSUSED*/
-static void
-parse_popt_args(poptContext con, enum poptCallbackReason reason,
-                const struct poptOption *opt, const char *arg, void *data)
+static gboolean
+pm_arguments_post_parse(GOptionContext *context, GOptionGroup *group,
+                        gpointer data, GError **error)
 {
+    gboolean meter_init = FALSE;
     int DL[PmMAX_METERS];
     int i, meter;
     long value;
-    gboolean meter_init = FALSE;
     gchar *charset;
-    char **argv, *s;
+    gchar **argv, *s;
     char *err = NULL;
 
-    /* Flag set when user passes  command line arguments */
-    cmd_line = TRUE;
+    if (opt_all) {
+        for (i = 0; i < PmMAX_METERS;i++) {
+            DL[i] = i;
+        }
 
-    switch (opt->shortName) {
-        case 'a':
-            for (i = 0; i < PmMAX_METERS;i++) {
-                DL[i] = i;
-            }
+        PMSETDISPLAYLIST(DL);
+    }
 
-            PMSETDISPLAYLIST(DL);
-            break;
+    if (opt_panel) {
+        PM->inFrontPanel = TRUE;
+        PM->ShowStripChart = FALSE;
+        PM->ShowLiveActionBar = TRUE;
+        PM->IsRemote = FALSE;
+    }
 
-        case 'f':
-            PM->inFrontPanel = TRUE;
-            PM->ShowStripChart = FALSE;
-            PM->ShowLiveActionBar = TRUE;
-            PM->IsRemote = FALSE;
-            break;
+    if (opt_log) {
+        PM->Log = TRUE;
+    }
 
-        case 'l':
-            PM->Log = TRUE;
-            break;
+    if (opt_logfile != NULL) {
+        PmStrcpy(PM->LogFilename, opt_logfile);
+        PM->Log = TRUE;
+    }
 
-        case 'n':
-            if (arg == NULL) {
-                Usage();
-            }
+    if (opt_pagelength != 0) {
+        PmAssignMin(opt_pagelength, PmMAX_VALUE);
+        PmAssignMax(opt_pagelength, 0);
+        PM->LogPageLength = opt_pagelength;
+        if (PM->LogPageLength < 1) {
+            PM->LogPageLength = PmPAGE_LENGTH;
+        }
 
-            PmStrcpy(PM->LogFilename, arg);
-            PM->Log = TRUE;
-            break;
+        PM->LogLineCount = PM->LogPageLength;
+    }
 
-        case 'p':
-/* XXX:richb - how we know there was an integer passed as 2nd arg? */
-            if (arg == NULL) {
-                Usage();
-            }
+    if (opt_sampletime != 0) {
+        PmAssignMin(opt_sampletime, PmMAX_SAMPLETIME);
+        PmAssignMax(opt_sampletime, 0);
+        PM->SampleInterval = opt_sampletime;
+        PM->SampleModifier = 1;
+    }
 
-            PmAssignMin(optTmpVal, PmMAX_VALUE);
-            PmAssignMax(optTmpVal, 0);
-            PM->LogPageLength = optTmpVal;
-            if (PM->LogPageLength < 1) {
-                PM->LogPageLength = PmPAGE_LENGTH;
-            }
+    if (opt_meter != NULL) {
+        /* Check for each meter name */
+        for (i = 0, meter = -1; i < PmMAX_METERS; i++) {
+            if (g_strcasecmp(PM->Meters[i].RName, opt_meter) == 0) {
+                meter = i;
+                break;
+            } else if (g_strcasecmp("colls", opt_meter) == 0) {
+                /* Check for colls separately; necessary for backwards
+                 * compatability.
+                 */
 
-            PM->LogLineCount = PM->LogPageLength;
-            break;
-
-        case 's':
-/* XXX:richb - how we know there was an integer passed as 2nd arg? */
-            if (arg == NULL) {
-                Usage();
-            }
-
-            PmAssignMin(optTmpVal, PmMAX_SAMPLETIME);
-            PmAssignMax(optTmpVal, 0);
-            PM->SampleInterval = optTmpVal;
-            PM->SampleModifier = 1;
-            break;
-
-        case 't':
-            if (arg == NULL) {
-                Usage();
-            }
- 
-            /* Check for each meter name */
-            for (i = 0, meter = -1; i < PmMAX_METERS; i++) {
-                if (strcasecmp(PM->Meters[i].RName, arg) == 0) {
-                    meter = i;
-                    break;
-                }
-            }
- 
-            /* Check for colls seperately: necessary for back compatability */
-            if (strcasecmp(arg, "colls") == 0) {
-                                  /* Get the meter number for coll */
+                /* Get the meter number for coll */
                 for (i = 0, meter =- 1; i < PmMAX_METERS; i++) {
-                    if (strcasecmp(PM->Meters[i].Name, "coll") == 0) {
+                    if (g_strcasecmp(PM->Meters[i].Name, "coll") == 0) {
                         meter = i;
                         break;
                     }
                 }
             }
+        }
  
-            if (meter < 0) {
-                Usage();
-            }
- 
-            if (!meter_init) {
- 
-                /* Clear all the meters */
-                PmLock();
-                meter_init = TRUE;
-                PM->NMeters = 0;
-                for (i = 0; i < PmMAX_METERS; i++) {
-                    PM->DisplayList[i] = -1;
-                    PM->Meters[i].Redisplay = FALSE;
-                    PM->Meters[i].Showing = FALSE;
-                }
-            }
-            PmResetMeter(meter, TRUE);
-            PmUnLock();
-            break;
-
-        case 'C':
-            if (arg == NULL) {
-                Usage();
-            }
-
-            argv = g_strsplit(arg, " ", 10);
-
-            /* Check if any of the token is NULL */
-            if ((argv[0] == NULL) || (argv[1] == NULL)) {
-                Usage();
-            } 
-
-            for (i = 0, meter =- 1; i < PmMAX_METERS; i++) {
-                if (strcasecmp(PM->Meters[i].Name, argv[0]) == 0) {
-                    meter = i;
-                    break;
-                }
-            }    
-            if (meter >= 0) {
-                value = strtol(argv[1], NULL, 10);
-                PmAssignMin(value, PmMAX_VALUE);
-                PmAssignMax(value, 0);
-                PM->Meters[meter].Limit = (int) value;
-            } else {
-                Usage();
-            }
-
-            if (!meter_init) {
-
-                /* Clear all the meters */
-                PmLock();
-                meter_init = TRUE;
-                PM->NMeters = 0;
-                for (i = 0; i < PmMAX_METERS; i++) {
-                    PM->DisplayList[i] = -1;
-                    PM->Meters[i].Redisplay = FALSE;
-                    PM->Meters[i].Showing = FALSE;
-                }
-            }
-            PmResetMeter(meter, TRUE);
-            PmUnLock();
-
-            g_strfreev(argv);
-            break;
-
-        case 'D':
-            PM->Debug = TRUE;
-            break;
-
-        case 'H':
-            PM->Orientation = PM_HORIZONTAL;
-            break;
-
-        case 'L':
-            PmSetGraphType(PM_LINE);
-            break;
-
-        case 'S':
-            PmSetGraphType(PM_SOLID);
-            break;
-
-        case 'v':
-            s = _("%s: version %s\n");
-            charset = g_locale_from_utf8(s, -1, NULL, NULL, NULL);
-            FPRINTF(stdout, charset, PM->AppClass, VERSION);
-            g_free(charset);
-            exit(1);
-            break;
-
-        case 'V':
-            PM->Orientation = PM_VERTICAL;
-            break;
-
-        case 'M':
-            if (arg == NULL) {
-                Usage();
-            }
-
-            argv = g_strsplit(arg, " ", 10);
-
-            /* Check if any of the token is NULL */
-            if ((argv[0] == NULL) || (argv[1] == NULL) ||
-                (argv[2] == NULL) || (argv[3] == NULL)) {
-                Usage();
-            }  
-
-            for (i = 0, meter =- 1; i < PmMAX_METERS; i++) {
-                if (strcasecmp(PM->Meters[i].Name, argv[0]) == 0) {
-                    meter = i;
-                    break;
-                }
-            }
- 
-            if (meter < 0) {
-                Usage();
-            }
-
-            value = strtol(argv[1], &err, 10);
-            if (*err || value < 0) {
-               Usage();
-            }
-
-            PmAssignMin(value, PmMAX_VALUE);
-            PmAssignMax(value, 0);
-            PM->Meters[meter].CurMax = (int) value;
-
-            value = strtol(argv[2], &err, 10);
-            if (*err || value < 0) {
-               Usage();
-            }
- 
-            PmAssignMin(value, PmMAX_VALUE);
-            PmAssignMax(value, 0);
-            PM->Meters[meter].MinMax = (int) value;
-
-            value = strtol(argv[3], &err, 10);
-            if (*err || value < 0) {
-               Usage();
-            }
-
-            PmAssignMin(value, PmMAX_VALUE);
-            PmAssignMax(value, 0);
-            PM->Meters[meter].MaxMax = (int) value;
-
-            if (!meter_init) {
-
-                /* Clear all the meters */
-                PmLock();
-                meter_init = TRUE;
-                PM->NMeters = 0;
-                for (i = 0; i < PmMAX_METERS; i++) {
-                    PM->DisplayList[i] = -1;
-                    PM->Meters[i].Redisplay = FALSE;
-                    PM->Meters[i].Showing = FALSE;
-                }
-            }
-            PmResetMeter(meter, TRUE);
-            PmUnLock();
-
-            g_strfreev(argv);
-            break;
-
-        default:
+        /* User passed invalid meter name */
+        if (meter < 0) {
             Usage();
+        }
+
+        if (!meter_init) {
+            /* Clear all the meters */
+            PmLock();
+            meter_init = TRUE;
+            PM->NMeters = 0;
+            for (i = 0; i < PmMAX_METERS; i++) {
+                PM->DisplayList[i] = -1;
+                PM->Meters[i].Redisplay = FALSE;
+                PM->Meters[i].Showing = FALSE;
+            }
+        }
+        PmResetMeter(meter, TRUE);
+        PmUnLock();
     }
+
+    if (opt_ceiling) {
+        argv = g_strsplit(opt_ceiling, " ", 10);
+
+        /* Check if any of the token is NULL */
+        if ((argv[0] == NULL) || (argv[1] == NULL)) {
+            Usage();
+        } 
+
+        for (i = 0, meter =- 1; i < PmMAX_METERS; i++) {
+            if (g_strcasecmp(PM->Meters[i].Name, argv[0]) == 0) {
+                meter = i;
+                break;
+            }
+        }    
+        if (meter >= 0) {
+            value = strtol(argv[1], NULL, 10);
+            PmAssignMin(value, PmMAX_VALUE);
+            PmAssignMax(value, 0);
+            PM->Meters[meter].Limit = (int) value;
+        } else {
+            Usage();
+        }
+
+        if (!meter_init) {
+            /* Clear all the meters */
+            PmLock();
+            meter_init = TRUE;
+            PM->NMeters = 0;
+            for (i = 0; i < PmMAX_METERS; i++) {
+                PM->DisplayList[i] = -1;
+                PM->Meters[i].Redisplay = FALSE;
+                PM->Meters[i].Showing = FALSE;
+            }
+        }
+        PmResetMeter(meter, TRUE);
+        PmUnLock();
+
+        g_strfreev(argv);
+    }
+
+    PM->Debug = opt_debug;
+
+    if (opt_horizontal) {
+        PM->Orientation = PM_HORIZONTAL;
+    }
+
+    if (opt_line) {
+        PmSetGraphType(PM_LINE);
+    }
+
+    if (opt_solid) {
+        PmSetGraphType(PM_SOLID);
+    }
+
+    if (opt_version) {
+        s = _("%s: version %s\n");
+        g_print(g_locale_from_utf8(s, -1, NULL, NULL, NULL), PM->AppClass,
+                VERSION);
+        exit(0);
+    }
+
+    if (opt_vertical) {
+        PM->Orientation = PM_VERTICAL;
+    }
+
+    if (opt_maxvalue) {
+        if (opt_maxvalue == NULL) {
+            Usage();
+        }
+
+        argv = g_strsplit(opt_maxvalue, " ", 10);
+
+        /* Check if any of the token is NULL */
+        if ((argv[0] == NULL) || (argv[1] == NULL) ||
+            (argv[2] == NULL) || (argv[3] == NULL)) {
+            Usage();
+        }  
+
+        for (i = 0, meter =- 1; i < PmMAX_METERS; i++) {
+            if (g_strcasecmp(PM->Meters[i].Name, argv[0]) == 0) {
+                meter = i;
+                break;
+            }
+        }
+ 
+        if (meter < 0) {
+            Usage();
+        }
+
+        value = strtol(argv[1], &err, 10);
+        if (*err || value < 0) {
+            Usage();
+        }
+
+        PmAssignMin(value, PmMAX_VALUE);
+        PmAssignMax(value, 0);
+        PM->Meters[meter].CurMax = (int) value;
+
+        value = strtol(argv[2], &err, 10);
+        if (*err || value < 0) {
+            Usage();
+        }
+
+        PmAssignMin(value, PmMAX_VALUE);
+        PmAssignMax(value, 0);
+        PM->Meters[meter].MinMax = (int) value;
+
+        value = strtol(argv[3], &err, 10);
+        if (*err || value < 0) {
+            Usage();
+        }
+
+        PmAssignMin(value, PmMAX_VALUE);
+        PmAssignMax(value, 0);
+        PM->Meters[meter].MaxMax = (int) value;
+
+        if (!meter_init) {
+
+            /* Clear all the meters */
+            PmLock();
+            meter_init = TRUE;
+            PM->NMeters = 0;
+            for (i = 0; i < PmMAX_METERS; i++) {
+                PM->DisplayList[i] = -1;
+                PM->Meters[i].Redisplay = FALSE;
+                PM->Meters[i].Showing = FALSE;
+            }
+        }
+        PmResetMeter(meter, TRUE);
+        PmUnLock();
+
+        g_strfreev(argv);
+    }
+
+    if (opt_hostnames && opt_hostnames[0] != NULL) {
+        PM->Hostname = g_strdup(opt_hostnames[0]);
+
+        if (g_strcasecmp(PM->Hostname, PM->LocalMachine) == 0) {
+            PM->IsRemote = FALSE;
+        } else {
+            PM->IsRemote = TRUE;
+        }
+    }
+
+    return(FALSE);
 }
 
 
@@ -769,49 +756,56 @@ parse_popt_args(poptContext con, enum poptCallbackReason reason,
 static void
 ParseCmdOptions(int argc, char **argv)
 {
+    int i;
 #ifdef HAVE_BIND_TEXTDOMAIN_CODESET
     const char *charset;
 #endif /*HAVE_BIND_TEXTDOMAIN_CODESET*/
 
     const char **args;
-    gboolean host = FALSE;
+    gboolean retval, host = FALSE;
+    GError *error = NULL;
     GnomeClient *client;    
-    int i;
-    ctx = poptGetContext(PACKAGE, argc, (const char **) argv, options, 0);
+    GOptionGroup *main_group;
 
 #ifdef HAVE_BIND_TEXTDOMAIN_CODESET
-    g_get_charset (&charset);
+    g_get_charset(&charset);
     BIND_TEXTDOMAIN_CODESET(GETTEXT_PACKAGE, charset);
 #endif /*HAVE_BIND_TEXTDOMAIN_CODESET*/
 
     gnome_program_init(PM->AppName, VERSION,
                        LIBGNOMEUI_MODULE,
                        argc, argv,
-                       GNOME_PARAM_POPT_TABLE, options,
-                       GNOME_PARAM_POPT_FLAGS, 0,
                        GNOME_PARAM_APP_DATADIR, DATADIR, NULL);    
 
+    /* Note if any options were passed */
+    if (argc >= 2) {
+        if (strlen(argv[1]) >=2 && argv[1][0] == '-') {
+            cmd_line = TRUE;
+        }
+    }
+
 #ifdef HAVE_BIND_TEXTDOMAIN_CODESET
     BIND_TEXTDOMAIN_CODESET(GETTEXT_PACKAGE, "UTF-8");
 #endif /*HAVE_BIND_TEXTDOMAIN_CODESET*/
 
-    args = poptGetArgs(ctx);
+    ctx = g_option_context_new("- monitor performance ");
 
-    if (args != NULL) {
-        i = 0;
-        while (args[i] != NULL) {
-            if (host == FALSE) {
-                PM->IsRemote = TRUE;
-                PmStrcpy(PM->Hostname, args[i]);
-                if (strcasecmp(PM->Hostname, PM->LocalMachine) == 0) {
-                    PM->IsRemote = FALSE;
-                }
-                host = TRUE;
-            } else {
-                Usage();
-            }
-        }
-    }
+#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
+    g_option_context_add_main_entries(ctx, entries, GETTEXT_PACKAGE);
+#else
+    g_option_context_add_main_entries(ctx, entries, NULL);
+#endif
+
+    /* Enable automatic generation of --help output */
+    g_option_context_set_help_enabled(ctx, TRUE);
+    g_option_context_set_ignore_unknown_options(ctx, FALSE);
+
+    /* Set pre and post parse hooks */
+    main_group = g_option_context_get_main_group(ctx);
+    g_option_group_set_parse_hooks(main_group, NULL, pm_arguments_post_parse);
+
+    retval = g_option_context_parse(ctx, &argc, &argv, &error);
+    g_assert(retval == FALSE);
 
     client = gnome_master_client();
     g_signal_connect(GTK_OBJECT(client), "save_yourself",
@@ -825,27 +819,32 @@ ParseCmdOptions(int argc, char **argv)
         PM->ShowStripChart = FALSE;
         PM->ShowLimit = FALSE;
     }
+
+    g_option_context_free(ctx);
 }
 
 
 GConfValue *
 GetGConfResource(char *host, char *meter, char *key)
 {
+    gchar *key_buf;
     GConfValue *result;
-    char key_buf[PmMAX_LINE];
 
     if (host && meter) {
-        SPRINTF(key_buf, "/apps/gperfmeter/%s/%s/%s", host, meter, key);
+        key_buf = g_strdup_printf("/apps/gperfmeter/%s/%s/%s", 
+                                  host, meter, key);
     } else if (host) {
-        SPRINTF(key_buf, "/apps/gperfmeter/%s/%s", host, key);
+        key_buf = g_strdup_printf("/apps/gperfmeter/%s/%s", host, key);
     } else if (meter) {
-        SPRINTF(key_buf, "/apps/gperfmeter/%s/%s", meter, key);
+        key_buf = g_strdup_printf("/apps/gperfmeter/%s/%s", meter, key);
     } else {
-        SPRINTF(key_buf, "/apps/gperfmeter/%s", key);
+        key_buf = g_strdup_printf("/apps/gperfmeter/%s", key);
     }
 
     result = gconf_client_get(PM->gconf_client, key_buf, NULL);
 
+    g_free(key_buf);
+
     return(result);
 }
 
@@ -854,18 +853,19 @@ void
 PutResource(char *host, char *meter, char *key, 
             ResType key_value_type, gpointer key_value)
 {
+    gchar *key_buf;
     GError *error = NULL;
-    char key_buf[PmMAX_LINE];
 
     /* Key formation */
     if (host && meter) {
-        SPRINTF(key_buf, "/apps/gperfmeter/%s/%s/%s", host, meter, key);
+        key_buf = g_strdup_printf("/apps/gperfmeter/%s/%s/%s", 
+                                  host, meter, key);
     } else if (host) {
-        SPRINTF(key_buf, "/apps/gperfmeter/%s/%s", host, key);
+        key_buf = g_strdup_printf("/apps/gperfmeter/%s/%s", host, key);
     } else if (meter) {
-        SPRINTF(key_buf, "/apps/gperfmeter/%s/%s", meter, key);
+        key_buf = g_strdup_printf("/apps/gperfmeter/%s/%s", meter, key);
     } else {
-        SPRINTF(key_buf, "/apps/gperfmeter/%s", key);
+        key_buf = g_strdup_printf("/apps/gperfmeter/%s", key);
     }
 
     /* Setting the key value. */
@@ -894,33 +894,10 @@ PutResource(char *host, char *meter, char *key,
         g_warning("Error in %s : %s", g_quark_to_string(error->domain),
                   error->message);
     }
-}
 
+    g_free(key_buf);
 
-/* Get a string representation of an int. */
-
-static char *
-IntToStr(int x)
-{
-    static char buf[20];
-
-    SPRINTF(buf, "%d", x);
-
-    return(buf);
-}
-
-
-/* Get an int representation from a string. */
-
-static gboolean 
-StrToInt(char *s, int* value)
-{
-    if (s) {
-        *value = strtol(s, NULL, 10);
-        return(TRUE);
-    } else {
-        return(FALSE);
-    }
+    return;
 }
 
 
@@ -930,8 +907,9 @@ void
 PmSaveResources()
 {
     int i;
-    char *hname, *mname, buf[PmMAX_LINE];
-    gchar *temp_buf;
+    char *hname, *mname;
+    gchar *temp_buf = NULL, *tmp_num = NULL;
+    GString *tmp;
 
     /* General Resources */
 
@@ -953,16 +931,15 @@ PmSaveResources()
     PutResource(hname, mname, "HostName", RES_STRING, PM->Hostname);
 
     for (i = 0; i < PmMAX_HOST_LIST_SIZE; i++) {
-        gchar resname[PmMAX_LINE];
         GList *item;
 
-        SPRINTF(resname, "HostName%1d", i);
+        temp_buf = g_strdup_printf("HostName%1d", i);
         if ((item = g_list_nth(remote_hosts, i)) != NULL) {
-            PutResource(hname, mname, resname, RES_STRING, item->data);
+            PutResource(hname, mname, temp_buf, RES_STRING, item->data);
         }
+        g_free(temp_buf);
     }
 
-    PutResource(hname, mname, "HostName", RES_STRING, PM->Hostname);
     PutResource(hname, mname, "ShowLimit", RES_BOOL, 
                 GINT_TO_POINTER(PM->ShowLimit));
     PutResource(hname, mname, "Log", RES_BOOL, GINT_TO_POINTER(PM->Log));
@@ -994,20 +971,27 @@ PmSaveResources()
     PutResource(hname, mname, "Direction", RES_STRING,
                 (PM->Orientation == PM_VERTICAL ? "vertical" : "horizontal"));
     
+    PutResource(hname, mname, "ShowHostName", RES_BOOL, 
+                GINT_TO_POINTER(PM->ShowHostName));
     PutResource(hname, mname, "ShowMenubar", RES_BOOL, 
                 GINT_TO_POINTER(PM->ShowMenubar));
     PutResource(hname, mname, "ShowStripChart", RES_BOOL, 
                 GINT_TO_POINTER(PM->ShowStripChart));
     PutResource(hname, mname, "ShowLiveActionBar", RES_BOOL, 
                 GINT_TO_POINTER(PM->ShowLiveActionBar));
-    PmStrcpy(buf, "");
+
+    tmp = g_string_new ("");
     for (i = 0; i < PM->NMeters; i++) {
         if (i > 0) {
-            STRCAT(buf, ",");
+            tmp = g_string_append(tmp, ",");
         }
-        STRCAT(buf, IntToStr(PM->DisplayList[i]));
+
+        tmp_num = g_strdup_printf("%d", PM->DisplayList[i]);
+        tmp = g_string_append(tmp, tmp_num);
+        g_free(tmp_num);
     }
-    PutResource(hname, mname, "DisplayList", RES_STRING, buf);
+    PutResource(hname, mname, "DisplayList", RES_STRING, tmp->str);
+    g_string_free(tmp, TRUE);
 
     /* Host data resources */
 
@@ -1038,7 +1022,7 @@ PmSaveResources()
 
 #ifdef DEBUG
     if (hname != NULL && mname != NULL) {
-        PRINTF("Save resource to: %s %s\n", hname, mname);
+        g_print("Save resource to: %s %s\n", hname, mname);
     }
 #endif
 }
@@ -1060,7 +1044,7 @@ PmResetResources(char *filename)
 
 #ifdef DEBUG
     if (filename != NULL) {
-        PRINTF("Reset attributes from: %s\n", filename);
+        g_print("Reset attributes from: %s\n", filename);
     }
 #endif
 
@@ -1089,9 +1073,8 @@ PmResetResources(char *filename)
     g_list_free(remote_hosts);
     remote_hosts = NULL;
     for (i = 0; i < PmMAX_HOST_LIST_SIZE; i++) {
-        gchar resname[PmMAX_LINE];
+        gchar *resname = g_strdup_printf("HostName%1d", i);
 
-        SPRINTF(resname, "HostName%1d", i);
         if ((gptr = GetGConfResource(hname, mname, resname))) {
             const char *data = gconf_value_get_string(gptr);
 
@@ -1102,10 +1085,11 @@ PmResetResources(char *filename)
                                              (GCompareFunc) g_ascii_strcasecmp);
             }
         }
+        g_free(resname);
     }
 
     if ((gptr = GetGConfResource(hname, mname, "HostName"))) {
-        SPRINTF(PM->Hostname, "%s", gconf_value_get_string(gptr));
+        PM->Hostname = g_strdup(gconf_value_get_string(gptr));
     }
 
     if (PM->LoadByHost) {
@@ -1125,7 +1109,7 @@ PmResetResources(char *filename)
     }
     
     if ((gptr = GetGConfResource(hname, mname, "LogFilename"))) {
-        SPRINTF(PM->LogFilename, "%s", gconf_value_get_string(gptr));
+        PM->LogFilename = g_strdup(gconf_value_get_string (gptr));
     }
 
     if ((gptr = GetGConfResource(hname, mname, "LogPageLength"))) {
@@ -1151,22 +1135,26 @@ PmResetResources(char *filename)
     if ((gptr = GetGConfResource(hname, mname, "GraphType"))) {
         const char * graph_type = gconf_value_get_string(gptr);
 
-        if (strcasecmp(graph_type, "line") == 0) {
+        if (g_strcasecmp(graph_type, "line") == 0) {
             PM->GraphType = PM_LINE;
-        } else if (strcasecmp(graph_type, "solid") == 0) {
+        } else if (g_strcasecmp(graph_type, "solid") == 0) {
             PM->GraphType = PM_SOLID;
         }
     }
     if ((gptr = GetGConfResource(hname, mname, "Direction"))) {
         const char * direction = gconf_value_get_string(gptr);
 
-        if (strcasecmp(direction, "vertical") == 0) {
+        if (g_strcasecmp(direction, "vertical") == 0) {
             PM->Orientation = PM_VERTICAL;
-        } else if (strcasecmp(direction, "horizontal") == 0) {
+        } else if (g_strcasecmp(direction, "horizontal") == 0) {
             PM->Orientation = PM_HORIZONTAL;
         }
     }
 
+    if ((gptr = GetGConfResource(hname, mname, "ShowHostName"))) {
+        PM->ShowHostName = gconf_value_get_bool(gptr);
+    }
+
     if ((gptr = GetGConfResource(hname, mname, "ShowMenubar"))) {
         PM->ShowMenubar = gconf_value_get_bool(gptr);
     }
@@ -1181,6 +1169,7 @@ PmResetResources(char *filename)
     
     if ((gptr = GetGConfResource(hname, mname, "DisplayList"))) {
         char *s;
+        gchar *dl;
         int display_list[PmMAX_METERS];
 
         for (i = 0; i < PmMAX_METERS; i++) {
@@ -1190,9 +1179,8 @@ PmResetResources(char *filename)
         PmStrcpy(buf, gconf_value_get_string(gptr));
         s = strtok(buf, " ,:");
         for (i = 0; i < PmMAX_METERS && s; i++) {
-            int value;
+            gdouble value = g_strtod(s, NULL);
 
-            STRTOINT(s, &value);
             if ((value >= 0) && (value < PmMAX_METERS)) {
                 display_list[i] = value;
             }
@@ -1237,7 +1225,7 @@ PmResetResources(char *filename)
 
     PmCheckLimits();
     PmSetScaleFactors();
-    PmExpandPathname(PM->LogFilename, PM->LogFilename);
+    PM->LogFilename = expand_tilde(PM->LogFilename);
     PmUnLock();
     PmResetProps();
 
@@ -1295,8 +1283,6 @@ Usage()
  *        -        - decrement the sample time by 1 second
  */
 
-    poptPrintHelp(ctx, stderr, 0);
-
     usage = _("\n\tmeter is one of:\n\
              cpu, packets, page, swap, interrupts, disk, context, load, collisions, errors\n\n\
              Keyboard accelerators:\n\n\
@@ -1325,7 +1311,7 @@ Usage()
              -      - decrement the sample time by 1 second\n");
 
     charset = g_locale_from_utf8(usage, -1, NULL, NULL, NULL);
-    FPRINTF(stderr, charset);
+    g_printerr (charset);
     g_free(charset);
 
     exit(1);
@@ -1334,93 +1320,64 @@ Usage()
 
 /*
  * Function
- *     PmExpandPathname
+ *     expand_tilde
  *
  * Description
- *     expand ~'s and environment variables in a path.
+ *     Takes a user-typed filename and expands a tilde at the beginning of
+ *     the string
  *
- *     this routine was stolen from filemgr. couple of small changes 
- *     were added to better handle "~user" at the start of a path.
+ *     This routine was stolen from GNOME CVS (beagle)
  *       
  * Parameters
- *     path    unexpanded path
- *     buf     returned expanded path. caller is responsible allocating 
- *             enough space.
- *       
+ *     filename    unexpanded path
+ *
  * Returns
- *     nothing
+ *     expanded path
+ *
  */
 
-void 
-PmExpandPathname(char *path, char * buf)
+gchar * 
+expand_tilde(const char *filename)
 {
-    register char *p, *b_p, *e_p ; /* string pointers */
-    char *save_p ;                 /* point in path before env var */
-    char env[MAXPATHLEN];          /* environment variable expansion */
-    struct passwd *pw;             /* password file entry */
+    const char *home, *notilde, *slash;
 
-    if (!path || !buf) {
-        return;
+    if (!filename) {
+        return(NULL);
     }
- 
-    p = path;
-    if (*p == '~') {
-        p++;
-        if (*p && *p != '/') {
 
-            /* Somebody else's home directory? */
-            if ((b_p = strchr(p, '/'))) {
-                *b_p = '\0';
-            }
+    if (filename[0] != '~') {
+        return(g_strdup(filename));
+    }
 
-            if ((pw = getpwnam(p))) {
-                STRCPY(buf, pw->pw_dir);
-            } else {
-                *buf = '~';
-                STRCPY(buf + 1, p);
-            }
- 
-            if (b_p) {
-                *b_p = '/';
-                p = b_p;
-            } else {
-                return;
-            }
-        } else {
-            STRCPY(buf, (char *) getenv("HOME"));
+    notilde = filename + 1;
+
+    slash = strchr(notilde, G_DIR_SEPARATOR);
+    if (!slash) {
+        return(NULL);
+    }
+
+    if (slash == notilde) {
+        home = g_get_home_dir();
+
+        if (!home) {
+            return(g_strdup(filename));
         }
-
-        b_p = buf + strlen(buf);
     } else {
-        b_p = buf;
-    }
+        char *username;
+        struct passwd *passwd;
 
-    while (*p) {
-        if (*p == '$') {
- 
-            /* expand environment variable */
-            save_p = p;
-            e_p = env;
-            p++;
-            while ((isalnum((int) *p) || *p == '_') && e_p - env < 255) {
-               *e_p++ = *p++;
-            }
+        username = g_strndup(notilde, slash - notilde);
+        passwd = getpwnam(username);
+        g_free(username);
 
-            *e_p = '\0';
-            if ((e_p = (char *) getenv(env))) {
-                while (*e_p) {
-                    *b_p++ = *e_p++;
-                }
-            } else {   
-                p = save_p ;
-                *b_p++ = *p++;
-            }
-        } else {
-            *b_p++ = *p++;
+        if (!passwd) {
+            return(g_strdup(filename));
         }
+
+        home = passwd->pw_dir;
     }
 
-    *b_p = '\0';
+    return(g_build_filename(home, G_DIR_SEPARATOR_S, slash + 1, NULL));
 }
 
 
@@ -1453,19 +1410,24 @@ client_die(GnomeClient *client, gpointer client_data)
 static char *
 color_to_string(const GdkColor *color)
 {
-    char *s;
-    char *ptr;
-
-    s = g_strdup_printf("#%2X%2X%2X",
-                        color->red / 256,
-                        color->green / 256,
-                        color->blue / 256);
-
-    for (ptr = s; *ptr; ptr++) {
-        if (*ptr == ' ') {
-            *ptr = '0';
-        }
-    }
+    char *s = g_strdup_printf("#%02X%02X%02X",
+                              color->red / 256,
+                              color->green / 256,
+                              color->blue / 256);
 
     return(s);
 }
+
+/* Generate a name for a temporary file */
+gchar *
+pm_log_create_tmp_filename(void)
+{
+    gchar *filename, *tmp;
+
+    tmp = g_strdup_printf("%s.log.%ld", PM->AppName, getpid());
+    filename = g_build_filename(g_get_tmp_dir(), 
+                                G_DIR_SEPARATOR_S, tmp, NULL);
+    g_free(tmp);
+
+    return(filename);
+}
